/*
 * Copyright (C) 2007 by Rui Dias <ruijdias@users.sourceforge.net>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or (at
 * your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

%module all

/**
 * out typemaps for basic types and arrays of basic types
 **/

%typemap(out) off_t = long;
%typemap(out) ssize_t = long;
%typemap(out) uint8_t = long;
%typemap(out) uint16_t = long;
%typemap(out) uint32_t = long;
%typemap(out) uint64_t = long;
%typemap(out) off_t[ANY] {
    int i;
    $result = PyList_New($1_dim0);
    for (i = 0; i < $1_dim0; i++)
    {
        PyObject *o = PyInt_FromLong((long) $1[i]);
        PyList_SetItem($result,i,o);
    }
}
%typemap(out) uint8_t[ANY] = off_t[ANY];
%typemap(out) uint16_t[ANY] = off_t[ANY];
%typemap(out) uint32_t[ANY] = off_t[ANY];
%typemap(out) uint32_t[ANY][ANY] {
    int i, j;
    $result = PyList_New($1_dim0);
    for (i = 0; i < $1_dim0; i++)
    {
        PyObject *l = PyList_New($1_dim1);
        for (j = 0; j < $1_dim1; j++)
        {
            PyObject *o = PyInt_FromLong((long) $1[i][j]);
            PyList_SetItem(l,j,o);
        }
        PyList_SetItem($result,i,l);
    }
}

/**
 * in typemaps for basic types and arrays of basic types
 **/

%typemap(in) off_t = long;
%typemap(in) uint8_t = long;
%typemap(in) uint16_t = long;
%typemap(in) uint32_t = long;
%typemap(in) uint64_t = long;
%typemap(in) off_t[ANY] (off_t temp[$1_dim0]) {
  int i;
  if (!PyList_Check($input)) {
    PyErr_SetString(PyExc_TypeError,"Expected a list");
    return NULL;
  }
  if (PyList_Size($input) != $1_dim0) {
    PyErr_SetString(PyExc_ValueError,"Size mismatch. Expected $1_dim0 elements");
    return NULL;
  }
  for (i = 0; i < $1_dim0; i++) {
    PyObject *o = PyList_GetItem($input,i);
    if (PyInt_Check(o)) {
      temp[i] = (off_t) PyInt_AsLong(o);
    } else {
      PyErr_SetString(PyExc_ValueError,"Sequence elements must be integers");
      return NULL;
    }
  }
  $1 = temp;
}
%typemap(in) uint8_t[ANY] (uint8_t temp[$1_dim0]) {
  int i;
  if (!PyList_Check($input)) {
    PyErr_SetString(PyExc_TypeError,"Expected a list");
    return NULL;
  }
  if (PyList_Size($input) != $1_dim0) {
    PyErr_SetString(PyExc_ValueError,"Size mismatch. Expected $1_dim0 elements");
    return NULL;
  }
  for (i = 0; i < $1_dim0; i++) {
    PyObject *o = PyList_GetItem($input,i);
    if (PyInt_Check(o)) {
      temp[i] = (uint8_t) PyInt_AsLong(o);
    } else {
      PyErr_SetString(PyExc_ValueError,"Sequence elements must be integers");
      return NULL;
    }
  }
  $1 = temp;
}
%typemap(in) uint16_t[ANY] (uint16_t temp[$1_dim0]) {
  int i;
  if (!PyList_Check($input)) {
    PyErr_SetString(PyExc_TypeError,"Expected a list");
    return NULL;
  }
  if (PyList_Size($input) != $1_dim0) {
    PyErr_SetString(PyExc_ValueError,"Size mismatch. Expected $1_dim0 elements");
    return NULL;
  }
  for (i = 0; i < $1_dim0; i++) {
    PyObject *o = PyList_GetItem($input,i);
    if (PyInt_Check(o)) {
      temp[i] = (uint16_t) PyInt_AsLong(o);
    } else {
      PyErr_SetString(PyExc_ValueError,"Sequence elements must be integers");
      return NULL;
    }
  }
  $1 = temp;
}
%typemap(in) uint32_t[ANY] (uint32_t temp[$1_dim0]) {
  int i;
  if (!PyList_Check($input)) {
    PyErr_SetString(PyExc_TypeError,"Expected a list");
    return NULL;
  }
  if (PyList_Size($input) != $1_dim0) {
    PyErr_SetString(PyExc_ValueError,"Size mismatch. Expected $1_dim0 elements");
    return NULL;
  }
  for (i = 0; i < $1_dim0; i++) {
    PyObject *o = PyList_GetItem($input,i);
    if (PyInt_Check(o)) {
      temp[i] = (uint32_t) PyInt_AsLong(o);
    } else {
      PyErr_SetString(PyExc_ValueError,"Sequence elements must be integers");
      return NULL;
    }
  }
  $1 = temp;
}
%typemap(in) uint32_t[ANY][ANY] (uint32_t temp[$1_dim0][$1_dim1]) {
  int i, j;
  if (!PyList_Check($input)) {
    PyErr_SetString(PyExc_TypeError,"Expected a list");
    return NULL;
  }
  if (PyList_Size($input) != $1_dim0) {
    PyErr_SetString(PyExc_ValueError,"Size mismatch. Expected $1_dim0 elements");
    return NULL;
  }
  for (i = 0; i < $1_dim0; i++) {
    PyObject *o = PyList_GetItem($input,i);
    if (!PyList_Check(o)) {
        PyErr_SetString(PyExc_TypeError,"Expected a list");
        return NULL;
    }
    if (PyList_Size(o) != $1_dim1) {
        PyErr_SetString(PyExc_ValueError,"Size mismatch. Expected $1_dim0 elements");
        return NULL;
    }
    for (j = 0; j < $1_dim1; j++) {
        PyObject *o2 = PyList_GetItem(o,j);
        if (PyInt_Check(o2)) {
            temp[i][j] = (uint32_t) PyInt_AsLong(o2);
        } else {
            PyErr_SetString(PyExc_ValueError,"Sequence elements must be integers");
        return NULL;
        }
    }
  }
  $1 = temp;
}

/**
 * memberin typemaps for arrays
 **/

%typemap(memberin) off_t[ANY] {
  int i;
  for (i = 0; i < $1_dim0; i++) {
      $1[i] = $input[i];
  }
}
%typemap(memberin) uint8_t[ANY] = off_t[ANY];
%typemap(memberin) uint16_t[ANY] = off_t[ANY];
%typemap(memberin) uint32_t[ANY] = off_t[ANY];
%typemap(memberin) audio_attr_t[ANY] = off_t[ANY];
%typemap(memberin) subp_attr_t[ANY] = off_t[ANY];
%typemap(memberin) multichannel_ext_t[ANY] = off_t[ANY];
%typemap(memberin) btni_t[ANY] = off_t[ANY];
%typemap(memberin) sml_agl_data_t[ANY] = off_t[ANY];
%typemap(memberin) uint32_t[ANY][ANY] {
  int i, j;
  for (i = 0; i < $1_dim0; i++) {
    for (j = 0; j < $1_dim1; j++) {
      $1[i][j] = $input[i][j];
    }
  }
}

/**
 * out typemaps for arrays of struct based types
 **/

%typemap(out) audio_attr_t[ANY] {
    int i;
    $result = PyList_New($1_dim0);
    for (i = 0; i < $1_dim0; i++)
    {
        PyObject *o = SWIG_NewPointerObj(SWIG_as_voidptr(&($1[i])), SWIGTYPE_p_audio_attr_t, 0 |  0 );
        PyList_SetItem($result,i,o);
    }
}

%typemap(out) subp_attr_t[ANY] {
    int i;
    $result = PyList_New($1_dim0);
    for (i = 0; i < $1_dim0; i++)
    {
        PyObject *o = SWIG_NewPointerObj(SWIG_as_voidptr(&($1[i])), SWIGTYPE_p_subp_attr_t, 0 |  0 );
        PyList_SetItem($result,i,o);
    }
}

%typemap(out) multichannel_ext_t[ANY] {
    int i;
    $result = PyList_New($1_dim0);
    for (i = 0; i < $1_dim0; i++)
    {
        PyObject *o = SWIG_NewPointerObj(SWIG_as_voidptr(&($1[i])), SWIGTYPE_p_multichannel_ext_t, 0 |  0 );
        PyList_SetItem($result,i,o);
    }
}

%typemap(out) btni_t[ANY] {
    int i;
    $result = PyList_New($1_dim0);
    for (i = 0; i < $1_dim0; i++)
    {
        PyObject *o = SWIG_NewPointerObj(SWIG_as_voidptr(&($1[i])), SWIGTYPE_p_btni_t, 0 |  0 );
        PyList_SetItem($result,i,o);
    }
}

%typemap(out) sml_pbi_t_vob_a[ANY] {
    int i;
    $result = PyList_New($1_dim0);
    for (i = 0; i < $1_dim0; i++)
    {
        PyObject *o = SWIG_NewPointerObj(SWIG_as_voidptr(&($1[i])), SWIGTYPE_p_sml_pbi_t_vob_a, 0 |  0 );
        PyList_SetItem($result,i,o);
    }
}

%typemap(out) sml_agl_data_t[ANY] {
    int i;
    $result = PyList_New($1_dim0);
    for (i = 0; i < $1_dim0; i++)
    {
        PyObject *o = SWIG_NewPointerObj(SWIG_as_voidptr(&($1[i])), SWIGTYPE_p_sml_agl_data_t, 0 |  0 );
        PyList_SetItem($result,i,o);
    }
}

/**
 * in typemaps for arrays of struct based types
 **/

%typemap(in) audio_attr_t[ANY] (audio_attr_t temp[$1_dim0]) {
    int i, ret;
    void *ptr;
    if (!PyList_Check($input)) {
        PyErr_SetString(PyExc_TypeError,"Expected a list");
        return NULL;
    }
    if (PyList_Size($input) != $1_dim0) {
        PyErr_SetString(PyExc_ValueError,"Size mismatch. Expected $1_dim0 elements");
        return NULL;
    }
    for (i = 0; i < $1_dim0; i++) {
        PyObject *o = PyList_GetItem($input, i);
        ret = SWIG_ConvertPtr(o, &ptr, SWIGTYPE_p_audio_attr_t, 0 |  0 );
        if (!SWIG_IsOK(ret)) {
            PyErr_SetString(PyExc_ValueError,"Sequence elements must of type audio_attr_t");
            return NULL;
        } else {
            temp[i] = *((audio_attr_t *) ptr);
        }
    }
    $1 = temp;
}

%typemap(in) subp_attr_t[ANY] (subp_attr_t temp[$1_dim0]) {
    int i, ret;
    void *ptr;
    if (!PyList_Check($input)) {
        PyErr_SetString(PyExc_TypeError,"Expected a list");
        return NULL;
    }
    if (PyList_Size($input) != $1_dim0) {
        PyErr_SetString(PyExc_ValueError,"Size mismatch. Expected $1_dim0 elements");
        return NULL;
    }
    for (i = 0; i < $1_dim0; i++) {
        PyObject *o = PyList_GetItem($input, i);
        ret = SWIG_ConvertPtr(o, &ptr, SWIGTYPE_p_subp_attr_t, 0 |  0 );
        if (!SWIG_IsOK(ret)) {
            PyErr_SetString(PyExc_ValueError,"Sequence elements must of type subp_attr_t");
            return NULL;
        } else {
            temp[i] = *((subp_attr_t *) ptr);
        }
    }
    $1 = temp;
}

%typemap(in) multichannel_ext_t[ANY] (multichannel_ext_t temp[$1_dim0]) {
    int i, ret;
    void *ptr;
    if (!PyList_Check($input)) {
        PyErr_SetString(PyExc_TypeError,"Expected a list");
        return NULL;
    }
    if (PyList_Size($input) != $1_dim0) {
        PyErr_SetString(PyExc_ValueError,"Size mismatch. Expected $1_dim0 elements");
        return NULL;
    }
    for (i = 0; i < $1_dim0; i++) {
        PyObject *o = PyList_GetItem($input, i);
        ret = SWIG_ConvertPtr(o, &ptr, SWIGTYPE_p_multichannel_ext_t, 0 |  0 );
        if (!SWIG_IsOK(ret)) {
            PyErr_SetString(PyExc_ValueError,"Sequence elements must of type multichannel_ext_t");
            return NULL;
        } else {
            temp[i] = *((multichannel_ext_t *) ptr);
        }
    }
    $1 = temp;
}

%typemap(in) btni_t[ANY] (btni_t temp[$1_dim0]) {
    int i, ret;
    void *ptr;
    if (!PyList_Check($input)) {
        PyErr_SetString(PyExc_TypeError,"Expected a list");
        return NULL;
    }
    if (PyList_Size($input) != $1_dim0) {
        PyErr_SetString(PyExc_ValueError,"Size mismatch. Expected $1_dim0 elements");
        return NULL;
    }
    for (i = 0; i < $1_dim0; i++) {
        PyObject *o = PyList_GetItem($input, i);
        ret = SWIG_ConvertPtr(o, &ptr, SWIGTYPE_p_btni_t, 0 |  0 );
        if (!SWIG_IsOK(ret)) {
            PyErr_SetString(PyExc_ValueError,"Sequence elements must of type btni_t");
            return NULL;
        } else {
            temp[i] = *((btni_t *) ptr);
        }
    }
    $1 = temp;
}

%typemap(in) sml_agl_data_t[ANY] (sml_agl_data_t temp[$1_dim0]) {
    int i, ret;
    void *ptr;
    if (!PyList_Check($input)) {
        PyErr_SetString(PyExc_TypeError,"Expected a list");
        return NULL;
    }
    if (PyList_Size($input) != $1_dim0) {
        PyErr_SetString(PyExc_ValueError,"Size mismatch. Expected $1_dim0 elements");
        return NULL;
    }
    for (i = 0; i < $1_dim0; i++) {
        PyObject *o = PyList_GetItem($input, i);
        ret = SWIG_ConvertPtr(o, &ptr, SWIGTYPE_p_sml_agl_data_t, 0 |  0 );
        if (!SWIG_IsOK(ret)) {
            PyErr_SetString(PyExc_ValueError,"Sequence elements must of type sml_agl_data_t");
            return NULL;
        } else {
            temp[i] = *((sml_agl_data_t *) ptr);
        }
    }
    $1 = temp;
}

/**
 * Includes
 **/

%{
#include <stdint.h>
%}
%include "cmd_print.i"
%include "dvd_reader.i"
%include "ifo_print.i"
%include "ifo_read.i"
%include "ifo_types.i"
%include "nav_print.i"
%include "nav_read.i"
%include "nav_types.i"


