/*
 * Copyright (C) 2007 by Rui Dias <ruijdias@users.sourceforge.net>
 *
 * This file contains code from the libdvdread project.
 *
 * libdvdread: Provides a simple foundation for reading DVD-Video images.
 * Copyright (C) 2000, 2001, 2002 Bj�rn Englund <d4bjorn@dtek.chalmers.se>,
 *                                H�kan Hjort <d95hjort@dtek.chalmers.se>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


/**
 * Module definition
 */
#ifdef DOC
%module(docstring="This module provides functions for reading the contents of IFO files.") ifo_read
#endif


/**
 * Includes
 */
%{
#include <dvdread/ifo_read.h>
%}


/**
 * ifoOpen
 */
%feature("autodoc", "ifoOpen(dvd, title) -> ifo_handle_t\n\n\
Opens an IFO and reads in all the data for the IFO file corresponding to the\n\
given title.  If title 0 is given, the video manager IFO file is read.\n\
Returns a handle to a completely parsed structure.") ifoOpen;
ifo_handle_t *ifoOpen(dvd_reader_t *, int );


/**
 * ifoOpenVMGI
 */
%feature("autodoc", "ifoOpenVMGI(dvd) -> ifo_handle_t\n\n\
Opens an IFO and reads in _only_ the vmgi_mat data.") ifoOpenVMGI;
ifo_handle_t *ifoOpenVMGI(dvd_reader_t *);


/**
 * ifoOpenVTSI
 */
%feature("autodoc", "ifoOpenVTSI(dvd, title) -> ifo_handle_t\n\n\
Opens an IFO and reads in _only_ the vtsi_mat data.") ifoOpenVTSI;
ifo_handle_t *ifoOpenVTSI(dvd_reader_t *, int);


/**
 * ifoClose
 */
%feature("autodoc", "ifoClose(ifofile)\n\n\
Cleans up the IFO information.  This will free all data allocated for the\n\
substructures.") ifoClose;
void ifoClose(ifo_handle_t *);


/**
 * The following functions are for reading only part of the VMGI/VTSI files.
 * Returns 1 if the data was successfully read and 0 on error.
 */

%feature("autodoc", "ifoRead_PLT_MAIT(ifofile) -> okay\n\n\
Read in the Parental Management Information table, filling the\n\
ifofile.ptl_mait structure and its substructures.  This data is only\n\
located in the video manager information file.  This fills the\n\
ifofile.ptl_mait structure and all its substructures.") ifoRead_PTL_MAIT;
int ifoRead_PTL_MAIT(ifo_handle_t *);

%feature("autodoc", "ifoRead_VTS_ATRT(ifofile) -> okay\n\n\
Read in the attribute table for the main menu vob, filling the\n\
ifofile.vts_atrt structure and its substructures.  Only located in the\n\
video manager information file.  This fills in the ifofile.vts_atrt\n\
structure and all its substructures.") ifoRead_VTS_ATRT;
int ifoRead_VTS_ATRT(ifo_handle_t *);

%feature("autodoc", "ifoRead_TT_SRPT(ifofile) -> okay\n\n\
Reads the title info for the main menu, filling the ifofile.tt_srpt\n\
structure and its substructures.  This data is only located in the video\n\
manager information file.  This structure is mandatory in the IFO file.") ifoRead_TT_SRPT;
int ifoRead_TT_SRPT(ifo_handle_t *);

%feature("autodoc", "ifoRead_VTS_PTT_SRPT(ifofile) -> okay\n\n\
Reads in the part of title search pointer table, filling the\n\
ifofile.vts_ptt_srpt structure and its substructures.  This data is only\n\
located in the video title set information file.  This structure is\n\
mandatory, and must be included in the VTSI file.") ifoRead_VTS_PTT_SRPT;
int ifoRead_VTS_PTT_SRPT(ifo_handle_t *);

%feature("autodoc", "ifoRead_FP_PGC(ifofile) -> okay\n\n\
Reads in the first play program chain data, filling the\n\
ifofile.first_play_pgc structure.  This data is only located in the video\n\
manager information file (VMGI).  This structure is optional.") ifoRead_FP_PGC;
int ifoRead_FP_PGC(ifo_handle_t *);

%feature("autodoc", "ifoRead_PGCIT(ifofile) -> okay\n\n\
Reads in the program chain information table for the video title set.  Fills\n\
in the ifofile.vts_pgcit structure and its substructures, which includes\n\
the data for each program chain in the set.  This data is only located in\n\
the video title set information file.  This structure is mandatory, and must\n\
be included in the VTSI file.") ifoRead_PGCIT;
int ifoRead_PGCIT(ifo_handle_t *);

%feature("autodoc", "ifoRead_PGCI_UT(ifofile) -> okay\n\n\
Reads in the menu PGCI unit table for the menu VOB.  For the video manager,\n\
this corresponds to the VIDEO_TS.VOB file, and for each title set, this\n\
corresponds to the VTS_XX_0.VOB file.  This data is located in both the\n\
video manager and video title set information files.  For VMGI files, this\n\
fills the ifofile.vmgi_pgci_ut structure and all its substructures.  For\n\
VTSI files, this fills the ifofile.vtsm_pgci_ut structure.") ifoRead_PGCI_UT;
int ifoRead_PGCI_UT(ifo_handle_t *);

%feature("autodoc", "ifoRead_VTS_TMAPT(ifofile) -> okay\n\n\
Reads in the VTS Time Map Table, this data is only located in the video\n\
title set information file.  This fills the ifofile.vts_tmapt structure\n\
and all its substructures.  When present enables VOBU level time-based\n\
seeking for One_Sequential_PGC_Titles.") ifoRead_VTS_TMAPT;
int ifoRead_VTS_TMAPT(ifo_handle_t *);

%feature("autodoc", "ifoRead_C_ADT(ifofile) -> okay\n\n\
Reads in the cell address table for the menu VOB.  For the video manager,\n\
this corresponds to the VIDEO_TS.VOB file, and for each title set, this\n\
corresponds to the VTS_XX_0.VOB file.  This data is located in both the\n\
video manager and video title set information files.  For VMGI files, this\n\
fills the ifofile.vmgm_c_adt structure and all its substructures.  For VTSI\n\
files, this fills the ifofile.vtsm_c_adt structure.") ifoRead_C_ADT;
int ifoRead_C_ADT(ifo_handle_t *);

%feature("autodoc", "ifoRead_TITLE_C_ADT(ifofile) -> okay\n\n\
Reads in the cell address table for the video title set corresponding to\n\
this IFO file.  This data is only located in the video title set information\n\
file.  This structure is mandatory, and must be included in the VTSI file.\n\
This call fills the ifofile.vts_c_adt structure and its substructures.") ifoRead_TITLE_C_ADT;
int ifoRead_TITLE_C_ADT(ifo_handle_t *);

%feature("autodoc", "ifoRead_VOBU_ADMAP(ifofile) -> okay\n\n\
Reads in the VOBU address map for the menu VOB.  For the video manager, this\n\
corresponds to the VIDEO_TS.VOB file, and for each title set, this\n\
corresponds to the VTS_XX_0.VOB file.  This data is located in both the\n\
video manager and video title set information files.  For VMGI files, this\n\
fills the ifofile.vmgm_vobu_admap structure and all its substructures.  For\n\
VTSI files, this fills the ifofile.vtsm_vobu_admap structure.") ifoRead_VOBU_ADMAP;
int ifoRead_VOBU_ADMAP(ifo_handle_t *);

%feature("autodoc", "ifoRead_TITLE_VOBU_ADMAP(ifofile) -> okay\n\n\
Reads in the VOBU address map for the associated video title set.  This data\n\
is only located in the video title set information file.  This structure is\n\
mandatory, and must be included in the VTSI file.  Fills the\n\
ifofile.vts_vobu_admap structure and its substructures.") ifoRead_TITLE_VOBU_ADMAP;
int ifoRead_TITLE_VOBU_ADMAP(ifo_handle_t *);

%feature("autodoc", "ifoRead_TXTDT_MGI(ifofile) -> okay\n\n\
Reads in the text data strings for the DVD.  Fills the ifofile.txtdt_mgi\n\
structure and all its substructures.  This data is only located in the video\n\
manager information file.  This structure is mandatory, and must be included\n\
in the VMGI file.") ifoRead_TXTDT_MGI;
int ifoRead_TXTDT_MGI(ifo_handle_t *);


/**
 * The following functions are used for freeing parsed sections of the
 * ifo_handle_t structure and the allocated substructures.  The free calls
 * below are safe:  they will not mind if you attempt to free part of an IFO
 * file which was not read in or which does not exist.
 */
%feature("autodoc", "0");
void ifoFree_PTL_MAIT(ifo_handle_t *ifofile);
void ifoFree_VTS_ATRT(ifo_handle_t *ifofile);
void ifoFree_TT_SRPT(ifo_handle_t *ifofile);
void ifoFree_VTS_PTT_SRPT(ifo_handle_t *ifofile);
void ifoFree_FP_PGC(ifo_handle_t *ifofile);
void ifoFree_PGCIT(ifo_handle_t *ifofile);
void ifoFree_PGCI_UT(ifo_handle_t *ifofile);
void ifoFree_VTS_TMAPT(ifo_handle_t *ifofile);
void ifoFree_C_ADT(ifo_handle_t *ifofile);
void ifoFree_TITLE_C_ADT(ifo_handle_t *ifofile);
void ifoFree_VOBU_ADMAP(ifo_handle_t *ifofile);
void ifoFree_TITLE_VOBU_ADMAP(ifo_handle_t *ifofile);
void ifoFree_TXTDT_MGI(ifo_handle_t *ifofile);

