/*
 * Copyright (C) 2007 by Rui Dias <ruijdias@users.sourceforge.net>
 *
 * This file contains code from the libdvdread project.
 *
 * libdvdread: Provides a simple foundation for reading DVD-Video images.
 * Copyright (C) 2000, 2001 Bjrn Englund <d4bjorn@dtek.chalmers.se>,
 *                          Hkan Hjort <d95hjort@dtek.chalmers.se>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


/**
 * Module definition
 */
#ifdef DOC
%module(docstring="This module contains the IFO structures") ifo_types
#endif
%feature("autodoc", "0");


/**
 * Includes
 */
%{
#include <dvdread/ifo_types.h>
%}


/**
 * Some necessary defines
 */
#undef ATTRIBUTE_PACKED
#undef PRAGMA_PACK_BEGIN 
#undef PRAGMA_PACK_END
#if defined(__GNUC__)
#if __GNUC__ > 2 || (__GNUC__ == 2 && __GNUC_MINOR__ >= 95)
#define ATTRIBUTE_PACKED __attribute__ ((packed))
#define PRAGMA_PACK 0
#endif
#endif
#if !defined(ATTRIBUTE_PACKED)
#define ATTRIBUTE_PACKED
#define PRAGMA_PACK 1
#endif
#if PRAGMA_PACK
#pragma pack(1)
#endif


/**
 * Renames
 */
/* Pointer (array) members */
%rename(_pre_cmds) pre_cmds;
%rename(_post_cmds) post_cmds;
%rename(_cell_cmds) cell_cmds;
%rename(_program_map) program_map;
%rename(_cell_playback) cell_playback;
%rename(_cell_position) cell_position;
%rename(_pgci_srp) pgci_srp;
%rename(_lu) lu;
%rename(_cell_adr_table) cell_adr_table;
%rename(_vobu_start_sectors) vobu_start_sectors;
%rename(_title) title;
%rename(_pf_ptl_mai) pf_ptl_mai;
%rename(_countries) countries;
%rename(_vts) vts;
%rename(_vts_atrt_offsets) vts_atrt_offsets;
%rename(_ptt) ptt;
%rename(_ttu_offset) ttu_offset;
%rename(_map_ent) map_ent;
%rename(_tmap) tmap;
%rename(_tmap_offset) tmap_offset;
/* Pointer (array) members accessors */
%rename(tmap) tmap_get;
%rename(tmap_offset) tmap_offset_get;
%rename(map_ent) map_ent_get;
%rename(title) title_get;
%rename(ttu_offset) ttu_offset_get;
%rename(ptt) ptt_get;
%rename(vts) vts_get;
%rename(vts_atrt_offsets) vts_atrt_offsets_get;
%rename(countries) countries_get;
%rename(pf_ptl_mai) pf_ptl_mai_get;
%rename(vobu_start_sectors) vobu_start_sectors_get;
%rename(cell_adr_table) cell_adr_table_get;
%rename(lu) lu_get;
%rename(pgci_srp) pgci_srp_get;
%rename(program_map) program_map_get;
%rename(cell_playback) cell_playback_get;
%rename(cell_position) cell_position_get;
%rename(pre_cmds) pre_cmds_get;
%rename(post_cmds) post_cmds_get;
%rename(cell_cmds) cell_cmds_get;


/**
 * Constants
 */
#define COMMAND_DATA_SIZE 8U
#define PGC_COMMAND_TBL_SIZE 8U
#define BLOCK_TYPE_NONE         0x0
#define BLOCK_TYPE_ANGLE_BLOCK  0x1
#define BLOCK_MODE_NOT_IN_BLOCK 0x0
#define BLOCK_MODE_FIRST_CELL   0x1
#define BLOCK_MODE_IN_BLOCK     0x2
#define BLOCK_MODE_LAST_CELL    0x3
#define PGC_SIZE 236U
#define PGCI_SRP_SIZE 8U
#define PGCIT_SIZE 8U
#define PGCI_LU_SIZE 8U
#define PGCI_UT_SIZE 8U
#define C_ADT_SIZE 8U
#define VOBU_ADMAP_SIZE 4U
#define TT_SRPT_SIZE 8U
#define PTL_MAIT_COUNTRY_SIZE 8U
#define PTL_MAIT_SIZE 8U
#define VTS_ATTRIBUTES_SIZE 542U
#define VTS_ATTRIBUTES_MIN_SIZE 356U
#define VTS_ATRT_SIZE 8U
#define TXTDT_LU_SIZE 8U
#define TXTDT_MGI_SIZE 20U
#define VTS_PTT_SRPT_SIZE 8U
#define VTS_TMAP_SIZE 4U
#define VTS_TMAPT_SIZE 8U


/**
 * Common
 *
 * The following structures are used in both the VMGI and VTSI.
 */


/**
 * dvd_time_t
 */
%feature("autodoc", "DVD Time Information.") dvd_time_t;
typedef struct {
  uint8_t hour;
  uint8_t minute;
  uint8_t second;
  uint8_t frame_u; /* The two high bits are the frame rate. */
} ATTRIBUTE_PACKED dvd_time_t;


/**
 * vm_cmd_t
 */
%feature("autodoc", "Type to store per-command data.") vm_cmd_t;
typedef struct {
  uint8_t bytes[8];
} ATTRIBUTE_PACKED vm_cmd_t;
%extend vm_cmd_t {
    void set_bytes(int index, uint8_t obj)
    {
        $self->bytes[index] = obj;
    }
};


/**
 * video_attr_t
 */
%feature("autodoc", "Video Attributes.") video_attr_t;
typedef struct {
#ifdef WORDS_BIGENDIAN
  unsigned int mpeg_version         : 2;
  unsigned int video_format         : 2;
  unsigned int display_aspect_ratio : 2;
  unsigned int permitted_df         : 2;
  
  unsigned int line21_cc_1          : 1;
  unsigned int line21_cc_2          : 1;
  unsigned int unknown1             : 1;
  unsigned int bit_rate             : 1;
  
  unsigned int picture_size         : 2;
  unsigned int letterboxed          : 1;
  unsigned int film_mode            : 1;
#else
  unsigned int permitted_df         : 2;
  unsigned int display_aspect_ratio : 2;
  unsigned int video_format         : 2;
  unsigned int mpeg_version         : 2;
  
  unsigned int film_mode            : 1;
  unsigned int letterboxed          : 1;
  unsigned int picture_size         : 2;
  
  unsigned int bit_rate             : 1;
  unsigned int unknown1             : 1;
  unsigned int line21_cc_2          : 1;
  unsigned int line21_cc_1          : 1;
#endif
} ATTRIBUTE_PACKED video_attr_t;


/**
 * audio_attr_t
 */
%feature("autodoc", "Audio Attributes.") audio_attr_t;
typedef struct {
#ifdef WORDS_BIGENDIAN
  unsigned int audio_format           : 3;
  unsigned int multichannel_extension : 1;
  unsigned int lang_type              : 2;
  unsigned int application_mode       : 2;
  
  unsigned int quantization           : 2;
  unsigned int sample_frequency       : 2;
  unsigned int unknown1               : 1;
  unsigned int channels               : 3;
#else
  unsigned int application_mode       : 2;
  unsigned int lang_type              : 2;
  unsigned int multichannel_extension : 1;
  unsigned int audio_format           : 3;
  
  unsigned int channels               : 3;
  unsigned int unknown1               : 1;
  unsigned int sample_frequency       : 2;
  unsigned int quantization           : 2;
#endif
  uint16_t lang_code;
  uint8_t  lang_extension;
  uint8_t  code_extension;
  uint8_t unknown3;
  union {
    struct {
#ifdef WORDS_BIGENDIAN
      unsigned int unknown4           : 1;
      unsigned int channel_assignment : 3;
      unsigned int version            : 2;
      unsigned int mc_intro           : 1; /* probably 0: true, 1:false */
      unsigned int mode               : 1; /* Karaoke mode 0: solo 1: duet */
#else
      unsigned int mode               : 1;
      unsigned int mc_intro           : 1;
      unsigned int version            : 2;
      unsigned int channel_assignment : 3;
      unsigned int unknown4           : 1;
#endif
    } ATTRIBUTE_PACKED karaoke;
    struct {
#ifdef WORDS_BIGENDIAN
      unsigned int unknown5           : 4;
      unsigned int dolby_encoded      : 1; /* suitable for surround decoding */
      unsigned int unknown6           : 3;
#else
      unsigned int unknown6           : 3;
      unsigned int dolby_encoded      : 1;
      unsigned int unknown5           : 4;
#endif
    } ATTRIBUTE_PACKED surround;
  } ATTRIBUTE_PACKED app_info;
} ATTRIBUTE_PACKED audio_attr_t;


/**
 * multichannel_ext_t
 */
%feature("autodoc", "MultiChannel Extension") multichannel_ext_t;
typedef struct {
#ifdef WORDS_BIGENDIAN
  unsigned int zero1      : 7;
  unsigned int ach0_gme   : 1;

  unsigned int zero2      : 7;
  unsigned int ach1_gme   : 1;

  unsigned int zero3      : 4;
  unsigned int ach2_gv1e  : 1;
  unsigned int ach2_gv2e  : 1;
  unsigned int ach2_gm1e  : 1;
  unsigned int ach2_gm2e  : 1;

  unsigned int zero4      : 4;
  unsigned int ach3_gv1e  : 1;
  unsigned int ach3_gv2e  : 1;
  unsigned int ach3_gmAe  : 1;
  unsigned int ach3_se2e  : 1;

  unsigned int zero5      : 4;
  unsigned int ach4_gv1e  : 1;
  unsigned int ach4_gv2e  : 1;
  unsigned int ach4_gmBe  : 1;
  unsigned int ach4_seBe  : 1;
#else
  unsigned int ach0_gme   : 1;
  unsigned int zero1      : 7;

  unsigned int ach1_gme   : 1;
  unsigned int zero2      : 7;

  unsigned int ach2_gm2e  : 1;
  unsigned int ach2_gm1e  : 1;
  unsigned int ach2_gv2e  : 1;
  unsigned int ach2_gv1e  : 1;
  unsigned int zero3      : 4;

  unsigned int ach3_se2e  : 1;
  unsigned int ach3_gmAe  : 1;
  unsigned int ach3_gv2e  : 1;
  unsigned int ach3_gv1e  : 1;
  unsigned int zero4      : 4;

  unsigned int ach4_seBe  : 1;
  unsigned int ach4_gmBe  : 1;
  unsigned int ach4_gv2e  : 1;
  unsigned int ach4_gv1e  : 1;
  unsigned int zero5      : 4;
#endif
  uint8_t zero6[19];
} ATTRIBUTE_PACKED multichannel_ext_t;
%extend multichannel_ext_t {
    void set_zero6(int index, uint8_t obj)
    {
        $self->zero6[index] = obj;
    }
};


/**
 * subp_attr_t
 */
%feature("autodoc", "Subpicture Attributes.") subp_attr_t;
typedef struct {
  /*
   * type: 0 not specified
   *       1 language
   *       2 other
   * coding mode: 0 run length
   *              1 extended
   *              2 other
   * language: indicates language if type == 1
   * lang extension: if type == 1 contains the lang extension
   */
#ifdef WORDS_BIGENDIAN
  unsigned int code_mode : 3;
  unsigned int zero1     : 3;
  unsigned int type      : 2;
#else
  unsigned int type      : 2;
  unsigned int zero1     : 3;
  unsigned int code_mode : 3;
#endif
  uint8_t  zero2;
  uint16_t lang_code;
  uint8_t  lang_extension;
  uint8_t  code_extension;
} ATTRIBUTE_PACKED subp_attr_t;


/**
 * pgc_command_tbl_t
 */
%feature("autodoc", "PGC Command Table.") pgc_command_tbl_t;
typedef struct {
  uint16_t nr_of_pre;
  uint16_t nr_of_post;
  uint16_t nr_of_cell;
  uint16_t last_byte;
  vm_cmd_t *pre_cmds;
  vm_cmd_t *post_cmds;
  vm_cmd_t *cell_cmds;
} ATTRIBUTE_PACKED pgc_command_tbl_t;
%extend pgc_command_tbl_t {
    vm_cmd_t pre_cmds_get(int index)
    {
        return $self->pre_cmds[index];
    }
    void set_pre_cmds(PyObject *obj)
    {
        int i, dim=0;
        int ret=0;
        void *ptr=0;
        vm_cmd_t *temp=0;
        if (!PyList_Check(obj)) {
            return;
        }
        dim = PyList_Size(obj);
        if (dim <= 0) return;
        temp = (vm_cmd_t *) malloc(dim * sizeof(vm_cmd_t));
        for (i = 0; i < dim; i++) {
            PyObject *o = PyList_GetItem(obj, i);
            ret = SWIG_ConvertPtr(o, &ptr, SWIGTYPE_p_vm_cmd_t, 0 |  0 );
            if (!SWIG_IsOK(ret)) {
                free(temp);
                return;
            } else {
                temp[i] = *((vm_cmd_t *) ptr);
            }
        }
        if ($self->pre_cmds)
            free($self->pre_cmds);
        $self->pre_cmds = temp;
    }
    vm_cmd_t post_cmds_get(int index)
    {
        return $self->post_cmds[index];
    }
    void set_post_cmds(PyObject *obj)
    {
        int i, dim=0;
        int ret=0;
        void *ptr=0;
        vm_cmd_t *temp=0;
        if (!PyList_Check(obj)) {
            return;
        }
        dim = PyList_Size(obj);
        if (dim <= 0) return;
        temp = (vm_cmd_t *) malloc(dim * sizeof(vm_cmd_t));
        for (i = 0; i < dim; i++) {
            PyObject *o = PyList_GetItem(obj, i);
            ret = SWIG_ConvertPtr(o, &ptr, SWIGTYPE_p_vm_cmd_t, 0 |  0 );
            if (!SWIG_IsOK(ret)) {
                free(temp);
                return;
            } else {
                temp[i] = *((vm_cmd_t *) ptr);
            }
        }
        if ($self->post_cmds)
            free($self->post_cmds);
        $self->post_cmds = temp;
    }
    vm_cmd_t cell_cmds_get(int index)
    {
        return $self->cell_cmds[index];
    }
    void set_cell_cmds(PyObject *obj)
    {
        int i, dim=0;
        int ret=0;
        void *ptr=0;
        vm_cmd_t *temp=0;
        if (!PyList_Check(obj)) {
            return;
        }
        dim = PyList_Size(obj);
        if (dim <= 0) return;
        temp = (vm_cmd_t *) malloc(dim * sizeof(vm_cmd_t));
        for (i = 0; i < dim; i++) {
            PyObject *o = PyList_GetItem(obj, i);
            ret = SWIG_ConvertPtr(o, &ptr, SWIGTYPE_p_vm_cmd_t, 0 |  0 );
            if (!SWIG_IsOK(ret)) {
                free(temp);
                return;
            } else {
                temp[i] = *((vm_cmd_t *) ptr);
            }
        }
        if ($self->cell_cmds)
            free($self->cell_cmds);
        $self->cell_cmds = temp;
    }
};


/**
 * PGC Program Map
 */
typedef uint8_t pgc_program_map_t; 


/**
 * cell_playback_t
 */
%feature("autodoc", "Cell Playback Information.") cell_playback_t;
typedef struct {
#ifdef WORDS_BIGENDIAN
  unsigned int block_mode       : 2;
  unsigned int block_type       : 2;
  unsigned int seamless_play    : 1;
  unsigned int interleaved      : 1;
  unsigned int stc_discontinuity: 1;
  unsigned int seamless_angle   : 1;
  
  unsigned int playback_mode    : 1;  /**< When set, enter StillMode after each VOBU */
  unsigned int restricted       : 1;  /**< ?? drop out of fastforward? */
  unsigned int unknown2         : 6;
#else
  unsigned int seamless_angle   : 1;
  unsigned int stc_discontinuity: 1;
  unsigned int interleaved      : 1;
  unsigned int seamless_play    : 1;
  unsigned int block_type       : 2;
  unsigned int block_mode       : 2;
  
  unsigned int unknown2         : 6;
  unsigned int restricted       : 1;
  unsigned int playback_mode    : 1;
#endif
  uint8_t still_time;
  uint8_t cell_cmd_nr;
  dvd_time_t playback_time;
  uint32_t first_sector;
  uint32_t first_ilvu_end_sector;
  uint32_t last_vobu_start_sector;
  uint32_t last_sector;
} ATTRIBUTE_PACKED cell_playback_t;


/**
 * cell_position_t
 */
%feature("autodoc", "Cell Position Information.") cell_position_t;
typedef struct {
  uint16_t vob_id_nr;
  uint8_t  zero_1;
  uint8_t  cell_nr;
} ATTRIBUTE_PACKED cell_position_t;


/**
 * user_ops_t
 */
%feature("autodoc", "User Operations.") user_ops_t;
typedef struct {
#ifdef WORDS_BIGENDIAN
  unsigned int zero                           : 7; /* 25-31 */
  unsigned int video_pres_mode_change         : 1; /* 24 */
  
  unsigned int karaoke_audio_pres_mode_change : 1; /* 23 */
  unsigned int angle_change                   : 1;
  unsigned int subpic_stream_change           : 1;
  unsigned int audio_stream_change            : 1;
  unsigned int pause_on                       : 1;
  unsigned int still_off                      : 1;
  unsigned int button_select_or_activate      : 1;
  unsigned int resume                         : 1; /* 16 */
  
  unsigned int chapter_menu_call              : 1; /* 15 */
  unsigned int angle_menu_call                : 1;
  unsigned int audio_menu_call                : 1;
  unsigned int subpic_menu_call               : 1;
  unsigned int root_menu_call                 : 1;
  unsigned int title_menu_call                : 1;
  unsigned int backward_scan                  : 1;
  unsigned int forward_scan                   : 1; /* 8 */
  
  unsigned int next_pg_search                 : 1; /* 7 */
  unsigned int prev_or_top_pg_search          : 1;
  unsigned int time_or_chapter_search         : 1;
  unsigned int go_up                          : 1;
  unsigned int stop                           : 1;
  unsigned int title_play                     : 1;
  unsigned int chapter_search_or_play         : 1;
  unsigned int title_or_time_play             : 1; /* 0 */
#else
  unsigned int video_pres_mode_change         : 1; /* 24 */
  unsigned int zero                           : 7; /* 25-31 */
  
  unsigned int resume                         : 1; /* 16 */
  unsigned int button_select_or_activate      : 1;
  unsigned int still_off                      : 1;
  unsigned int pause_on                       : 1;
  unsigned int audio_stream_change            : 1;
  unsigned int subpic_stream_change           : 1;
  unsigned int angle_change                   : 1;
  unsigned int karaoke_audio_pres_mode_change : 1; /* 23 */
  
  unsigned int forward_scan                   : 1; /* 8 */
  unsigned int backward_scan                  : 1;
  unsigned int title_menu_call                : 1;
  unsigned int root_menu_call                 : 1;
  unsigned int subpic_menu_call               : 1;
  unsigned int audio_menu_call                : 1;
  unsigned int angle_menu_call                : 1;
  unsigned int chapter_menu_call              : 1; /* 15 */
  
  unsigned int title_or_time_play             : 1; /* 0 */
  unsigned int chapter_search_or_play         : 1;
  unsigned int title_play                     : 1;
  unsigned int stop                           : 1;
  unsigned int go_up                          : 1;
  unsigned int time_or_chapter_search         : 1;
  unsigned int prev_or_top_pg_search          : 1;
  unsigned int next_pg_search                 : 1; /* 7 */
#endif
} ATTRIBUTE_PACKED user_ops_t;


/**
 * pgc_t
 */
%feature("autodoc", "Program Chain Information.") pgc_t;
typedef struct {
  uint16_t zero_1;
  uint8_t  nr_of_programs;
  uint8_t  nr_of_cells;
  dvd_time_t playback_time;
  user_ops_t prohibited_ops;
  uint16_t audio_control[8]; /* New type? */
  uint32_t subp_control[32]; /* New type? */
  uint16_t next_pgc_nr;
  uint16_t prev_pgc_nr;
  uint16_t goup_pgc_nr;
  uint8_t  pg_playback_mode;
  uint8_t  still_time;
  uint32_t palette[16]; /* New type struct {zero_1, Y, Cr, Cb} ? */
  uint16_t command_tbl_offset;
  uint16_t program_map_offset;
  uint16_t cell_playback_offset;
  uint16_t cell_position_offset;
  pgc_command_tbl_t *command_tbl;
  pgc_program_map_t  *program_map;
  cell_playback_t *cell_playback;
  cell_position_t *cell_position;
} ATTRIBUTE_PACKED pgc_t;
%extend pgc_t {
    void set_audio_control(int index, uint16_t obj)
    {
        $self->audio_control[index] = obj;
    }
    void set_subp_control(int index, uint32_t obj)
    {
        $self->subp_control[index] = obj;
    }
    void set_palette(int index, uint32_t obj)
    {
        $self->palette[index] = obj;
    }
    pgc_program_map_t program_map_get(int index)
    {
        return $self->program_map[index];
    }
    void set_program_map(PyObject *obj)
    {
        int i, dim=0;
        int ret=0;
        uint8_t *temp=0;
        if (!PyList_Check(obj)) {
            return;
        }
        dim = PyList_Size(obj);
        if (dim <= 0) return;
        temp = (uint8_t *) malloc(dim * sizeof(uint8_t));
        for (i = 0; i < dim; i++) {
            PyObject *o = PyList_GetItem(obj, i);
            if (PyInt_Check(o)) {
                temp[i] = (uint8_t) PyInt_AsLong(o);
            } else {
                free(temp);
                return;
            }
        }
        if ($self->program_map)
            free($self->program_map);
        $self->program_map = temp;
    }
    cell_playback_t cell_playback_get(int index)
    {
        return $self->cell_playback[index];
    }
    void set_cell_playback(PyObject *obj)
    {
        int i, dim=0;
        int ret=0;
        void *ptr=0;
        cell_playback_t *temp=0;
        if (!PyList_Check(obj)) {
            return;
        }
        dim = PyList_Size(obj);
        if (dim <= 0) return;
        temp = (cell_playback_t *) malloc(dim * sizeof(cell_playback_t));
        for (i = 0; i < dim; i++) {
            PyObject *o = PyList_GetItem(obj, i);
            ret = SWIG_ConvertPtr(o, &ptr, SWIGTYPE_p_cell_playback_t, 0 |  0 );
            if (!SWIG_IsOK(ret)) {
                free(temp);
                return;
            } else {
                temp[i] = *((cell_playback_t *) ptr);
            }
        }
        if ($self->cell_playback)
            free($self->cell_playback);
        $self->cell_playback = temp;
    }
    cell_position_t cell_position_get(int index)
    {
        return $self->cell_position[index];
    }
    void set_cell_position(PyObject *obj)
    {
        int i, dim=0;
        int ret=0;
        void *ptr=0;
        cell_position_t *temp=0;
        if (!PyList_Check(obj)) {
            return;
        }
        dim = PyList_Size(obj);
        if (dim <= 0) return;
        temp = (cell_position_t *) malloc(dim * sizeof(cell_position_t));
        for (i = 0; i < dim; i++) {
            PyObject *o = PyList_GetItem(obj, i);
            ret = SWIG_ConvertPtr(o, &ptr, SWIGTYPE_p_cell_position_t, 0 |  0 );
            if (!SWIG_IsOK(ret)) {
                free(temp);
                return;
            } else {
                temp[i] = *((cell_position_t *) ptr);
            }
        }
        if ($self->cell_position)
            free($self->cell_position);
        $self->cell_position = temp;
    }
};


/**
 * pgci_srp_t
 */
%feature("autodoc", "Program Chain Information Search Pointer.") pgci_srp_t;
typedef struct {
  uint8_t  entry_id;
#ifdef WORDS_BIGENDIAN
  unsigned int block_mode : 2;
  unsigned int block_type : 2;
  unsigned int unknown1   : 4;
#else
  unsigned int unknown1   : 4;
  unsigned int block_type : 2;
  unsigned int block_mode : 2;
#endif  
  uint16_t ptl_id_mask;
  uint32_t pgc_start_byte;
  pgc_t *pgc;
} ATTRIBUTE_PACKED pgci_srp_t;


/**
 * pgcit_t
 */
%feature("autodoc", "Program Chain Information Table.") pgcit_t;
typedef struct {
  uint16_t nr_of_pgci_srp;
  uint16_t zero_1;
  uint32_t last_byte;
  pgci_srp_t *pgci_srp;
} ATTRIBUTE_PACKED pgcit_t;
%extend pgcit_t {
    pgci_srp_t pgci_srp_get(int index)
    {
        return $self->pgci_srp[index];
    }
    void set_pgci_srp(PyObject *obj)
    {
        int i, dim=0;
        int ret=0;
        void *ptr=0;
        pgci_srp_t *temp=0;
        if (!PyList_Check(obj)) {
            return;
        }
        dim = PyList_Size(obj);
        if (dim <= 0) return;
        temp = (pgci_srp_t *) malloc(dim * sizeof(pgci_srp_t));
        for (i = 0; i < dim; i++) {
            PyObject *o = PyList_GetItem(obj, i);
            ret = SWIG_ConvertPtr(o, &ptr, SWIGTYPE_p_pgci_srp_t, 0 |  0 );
            if (!SWIG_IsOK(ret)) {
                free(temp);
                return;
            } else {
                temp[i] = *((pgci_srp_t *) ptr);
            }
        }
        if ($self->pgci_srp)
            free($self->pgci_srp);
        $self->pgci_srp = temp;
    }
};


/**
 * pgci_lu_t
 */
%feature("autodoc", "Menu PGCI Language Unit.") pgci_lu_t;
typedef struct {
  uint16_t lang_code;
  uint8_t  lang_extension;
  uint8_t  exists;
  uint32_t lang_start_byte;
  pgcit_t *pgcit;
} ATTRIBUTE_PACKED pgci_lu_t;


/**
 * pgci_ut_t
 */
%feature("autodoc", "Menu PGCI Unit Table.") pgci_ut_t;
typedef struct {
  uint16_t nr_of_lus;
  uint16_t zero_1;
  uint32_t last_byte;
  pgci_lu_t *lu;
} ATTRIBUTE_PACKED pgci_ut_t;
%extend pgci_ut_t {
    pgci_lu_t lu_get(int index)
    {
        return $self->lu[index];
    }
    void set_lu(PyObject *obj)
    {
        int i, dim=0;
        int ret=0;
        void *ptr=0;
        pgci_lu_t *temp=0;
        if (!PyList_Check(obj)) {
            return;
        }
        dim = PyList_Size(obj);
        if (dim <= 0) return;
        temp = (pgci_lu_t *) malloc(dim * sizeof(pgci_lu_t));
        for (i = 0; i < dim; i++) {
            PyObject *o = PyList_GetItem(obj, i);
            ret = SWIG_ConvertPtr(o, &ptr, SWIGTYPE_p_pgci_lu_t, 0 |  0 );
            if (!SWIG_IsOK(ret)) {
                free(temp);
                return;
            } else {
                temp[i] = *((pgci_lu_t *) ptr);
            }
        }
        if ($self->lu)
            free($self->lu);
        $self->lu = temp;
    }
};


/**
 * cell_adr_t
 */
%feature("autodoc", "Cell Address Information.") cell_adr_t;
typedef struct {
  uint16_t vob_id;
  uint8_t  cell_id;
  uint8_t  zero_1;
  uint32_t start_sector;
  uint32_t last_sector;
} ATTRIBUTE_PACKED cell_adr_t;


/**
 * c_adt_t
 */
%feature("autodoc", "Cell Address Table.") c_adt_t;
typedef struct {
  uint16_t nr_of_vobs; /* VOBs */
  uint16_t zero_1;
  uint32_t last_byte;
  cell_adr_t *cell_adr_table;  /* No explicit size given. */
} ATTRIBUTE_PACKED c_adt_t;
%extend c_adt_t {
    cell_adr_t cell_adr_table_get(int index)
    {
        return $self->cell_adr_table[index];
    }
    void set_cell_adr_table(PyObject *obj)
    {
        int i, dim=0;
        int ret=0;
        void *ptr=0;
        cell_adr_t *temp=0;
        if (!PyList_Check(obj)) {
            return;
        }
        dim = PyList_Size(obj);
        if (dim <= 0) return;
        temp = (cell_adr_t *) malloc(dim * sizeof(cell_adr_t));
        for (i = 0; i < dim; i++) {
            PyObject *o = PyList_GetItem(obj, i);
            ret = SWIG_ConvertPtr(o, &ptr, SWIGTYPE_p_cell_adr_t, 0 |  0 );
            if (!SWIG_IsOK(ret)) {
                free(temp);
                return;
            } else {
                temp[i] = *((cell_adr_t *) ptr);
            }
        }
        if ($self->cell_adr_table)
            free($self->cell_adr_table);
        $self->cell_adr_table = temp;
    }
};


/**
 * vobu_admap_t
 */
%feature("autodoc", "VOBU Address Map.") vobu_admap_t;
typedef struct {
  uint32_t last_byte;
  uint32_t *vobu_start_sectors;
} ATTRIBUTE_PACKED vobu_admap_t;
%extend vobu_admap_t {
    uint32_t vobu_start_sectors_get(int index)
    {
        return $self->vobu_start_sectors[index];
    }
    void set_vobu_start_sectors(PyObject *obj)
    {
        int i, dim=0;
        int ret=0;
        uint32_t *temp=0;
        if (!PyList_Check(obj)) {
            return;
        }
        dim = PyList_Size(obj);
        if (dim <= 0) return;
        temp = (uint32_t *) malloc(dim * sizeof(uint32_t));
        for (i = 0; i < dim; i++) {
            PyObject *o = PyList_GetItem(obj, i);
            if (PyInt_Check(o)) {
                temp[i] = (uint32_t) PyInt_AsLong(o);
            } else {
                free(temp);
                return;
            }
        }
        if ($self->vobu_start_sectors)
            free($self->vobu_start_sectors);
        $self->vobu_start_sectors = temp;
    }
};


/**
 * VMGI
 *
 * The following structures relate to the Video Manager.
 */


/**
 * vmgi_mat_t
 */
%feature("autodoc", "Video Manager Information Management Table.") vmgi_mat_t;
typedef struct {
  char     vmg_identifier[12];
  uint32_t vmg_last_sector;
  uint8_t  zero_1[12];
  uint32_t vmgi_last_sector;
  uint8_t  zero_2;
  uint8_t  specification_version;
  uint32_t vmg_category;
  uint16_t vmg_nr_of_volumes;
  uint16_t vmg_this_volume_nr;
  uint8_t  disc_side;
  uint8_t  zero_3[19];
  uint16_t vmg_nr_of_title_sets;  /* Number of VTSs. */
  char     provider_identifier[32];
  uint64_t vmg_pos_code;
  uint8_t  zero_4[24];
  uint32_t vmgi_last_byte;
  uint32_t first_play_pgc;
  uint8_t  zero_5[56];
  uint32_t vmgm_vobs;             /* sector */
  uint32_t tt_srpt;               /* sector */
  uint32_t vmgm_pgci_ut;          /* sector */
  uint32_t ptl_mait;              /* sector */
  uint32_t vts_atrt;              /* sector */
  uint32_t txtdt_mgi;             /* sector */
  uint32_t vmgm_c_adt;            /* sector */
  uint32_t vmgm_vobu_admap;       /* sector */
  uint8_t  zero_6[32];
  
  video_attr_t vmgm_video_attr;
  uint8_t  zero_7;
  uint8_t  nr_of_vmgm_audio_streams; /* should be 0 or 1 */
  audio_attr_t vmgm_audio_attr;
  audio_attr_t zero_8[7];
  uint8_t  zero_9[17];
  uint8_t  nr_of_vmgm_subp_streams; /* should be 0 or 1 */
  subp_attr_t  vmgm_subp_attr;
  subp_attr_t  zero_10[27];  /* XXX: how much 'padding' here? */
} ATTRIBUTE_PACKED vmgi_mat_t;
%extend vmgi_mat_t {
    void set_zero_1(int index, uint8_t obj)
    {
        $self->zero_1[index] = obj;
    }
    void set_zero_3(int index, uint8_t obj)
    {
        $self->zero_3[index] = obj;
    }
    void set_zero_4(int index, uint8_t obj)
    {
        $self->zero_4[index] = obj;
    }
    void set_zero_5(int index, uint8_t obj)
    {
        $self->zero_5[index] = obj;
    }
    void set_zero_6(int index, uint8_t obj)
    {
        $self->zero_6[index] = obj;
    }
    void set_zero_8(int index, audio_attr_t obj)
    {
        $self->zero_8[index] = obj;
    }
    void set_zero_9(int index, uint8_t obj)
    {
        $self->zero_9[index] = obj;
    }
    void set_zero_10(int index, subp_attr_t obj)
    {
        $self->zero_10[index] = obj;
    }
};


/**
 * playback_type_t
 */
%feature("autodoc", "Playback Type.") playback_type_t;
typedef struct {
#ifdef WORDS_BIGENDIAN
  unsigned int zero_1                    : 1;
  unsigned int multi_or_random_pgc_title : 1; /* 0: one sequential pgc title */
  unsigned int jlc_exists_in_cell_cmd    : 1;
  unsigned int jlc_exists_in_prepost_cmd : 1;
  unsigned int jlc_exists_in_button_cmd  : 1;
  unsigned int jlc_exists_in_tt_dom      : 1;
  unsigned int chapter_search_or_play    : 1; /* UOP 1 */
  unsigned int title_or_time_play        : 1; /* UOP 0 */
#else
  unsigned int title_or_time_play        : 1;
  unsigned int chapter_search_or_play    : 1;
  unsigned int jlc_exists_in_tt_dom      : 1;
  unsigned int jlc_exists_in_button_cmd  : 1;
  unsigned int jlc_exists_in_prepost_cmd : 1;
  unsigned int jlc_exists_in_cell_cmd    : 1;
  unsigned int multi_or_random_pgc_title : 1;
  unsigned int zero_1                    : 1;
#endif
} ATTRIBUTE_PACKED playback_type_t;


/**
 * title_info_t
 */
%feature("autodoc", "Title Information.") title_info_t;
typedef struct {
  playback_type_t pb_ty;
  uint8_t  nr_of_angles;
  uint16_t nr_of_ptts;
  uint16_t parental_id;
  uint8_t  title_set_nr;
  uint8_t  vts_ttn;
  uint32_t title_set_sector;
} ATTRIBUTE_PACKED title_info_t;


/**
 * tt_srpt_t
 */
%feature("autodoc", "PartOfTitle Search Pointer Table.") tt_srpt_t;
typedef struct {
  uint16_t nr_of_srpts;
  uint16_t zero_1;
  uint32_t last_byte;
  title_info_t *title;
} ATTRIBUTE_PACKED tt_srpt_t;
%extend tt_srpt_t {
    title_info_t title_get(int index)
    {
        return $self->title[index];
    }
    void set_title(PyObject *obj)
    {
        int i, dim=0;
        int ret=0;
        void *ptr=0;
        title_info_t *temp=0;
        if (!PyList_Check(obj)) {
            return;
        }
        dim = PyList_Size(obj);
        if (dim <= 0) return;
        temp = (title_info_t *) malloc(dim * sizeof(title_info_t));
        for (i = 0; i < dim; i++) {
            PyObject *o = PyList_GetItem(obj, i);
            ret = SWIG_ConvertPtr(o, &ptr, SWIGTYPE_p_title_info_t, 0 |  0 );
            if (!SWIG_IsOK(ret)) {
                free(temp);
                return;
            } else {
                temp[i] = *((title_info_t *) ptr);
            }
        }
        if ($self->title)
            free($self->title);
        $self->title = temp;
    }
};


/**
 * Parental Management Information Unit Table.
 * Level 1 (US: G), ..., 7 (US: NC-17), 8
 */
typedef uint16_t pf_level_t[8];


/**
 * ptl_mait_country_t
 */
%feature("autodoc", "Parental Management Information Unit Table.") ptl_mait_country_t;
typedef struct {
  uint16_t country_code;
  uint16_t zero_1;
  uint16_t pf_ptl_mai_start_byte;
  uint16_t zero_2;
  pf_level_t *pf_ptl_mai; /* table of (nr_of_vtss + 1), video_ts is first */
} ATTRIBUTE_PACKED ptl_mait_country_t;
%extend ptl_mait_country_t {
    PyObject *pf_ptl_mai_get(int index)
    {
        PyObject *o = PyList_New(8);
        int i;
        for (i = 0; i < 8; i++)
            PyList_SetItem(o, i, PyInt_FromLong($self->pf_ptl_mai[index][i]));
        return o;
    }
    void set_pf_ptl_mai(PyObject *obj)
    {
        int i, j, dim=0;
        int ret=0;
        pf_level_t *temp=0;
        if (!PyList_Check(obj)) {
            return;
        }
        dim = PyList_Size(obj);
        if (dim <= 0) return;
        temp = (pf_level_t *) malloc(dim * sizeof(pf_level_t));
        for (i = 0; i < dim; i++) {
            PyObject *o = PyList_GetItem(obj, i);
            if (!PyList_Check(o) || PyList_Size(o) != 8) {
                free(temp);
                return;
            } else {
                for (j = 0; j < 8; j++) {
                    PyObject *o2 = PyList_GetItem(o, j);
                    if (PyInt_Check(o2)) {
                        temp[i][j] = (uint16_t) PyInt_AsLong(o2);
                    } else {
                        free(temp);
                        return;
                    }
                }
            }
        }
        if ($self->pf_ptl_mai)
            free($self->pf_ptl_mai);
        $self->pf_ptl_mai = temp;
    }
};


/**
 * ptl_mait_t
 */
%feature("autodoc", "Parental Management Information Table.") ptl_mait_t;
typedef struct {
  uint16_t nr_of_countries;
  uint16_t nr_of_vtss;
  uint32_t last_byte;
  ptl_mait_country_t *countries;
} ATTRIBUTE_PACKED ptl_mait_t;
%extend ptl_mait_t {
    ptl_mait_country_t countries_get(int index)
    {
        return $self->countries[index];
    }
    void set_countries(PyObject *obj)
    {
        int i, dim=0;
        int ret=0;
        void *ptr=0;
        ptl_mait_country_t *temp=0;
        if (!PyList_Check(obj)) {
            return;
        }
        dim = PyList_Size(obj);
        if (dim <= 0) return;
        temp = (ptl_mait_country_t *) malloc(dim * sizeof(ptl_mait_country_t));
        for (i = 0; i < dim; i++) {
            PyObject *o = PyList_GetItem(obj, i);
            ret = SWIG_ConvertPtr(o, &ptr, SWIGTYPE_p_ptl_mait_country_t, 0 |  0 );
            if (!SWIG_IsOK(ret)) {
                free(temp);
                return;
            } else {
                temp[i] = *((ptl_mait_country_t *) ptr);
            }
        }
        if ($self->countries)
            free($self->countries);
        $self->countries = temp;
    }
};


/**
 * vts_attributes_t
 */
%feature("autodoc", "Video Title Set Attributes.") vts_attributes_t;
typedef struct {
  uint32_t last_byte;
  uint32_t vts_cat;
  
  video_attr_t vtsm_vobs_attr;
  uint8_t  zero_1;
  uint8_t  nr_of_vtsm_audio_streams; /* should be 0 or 1 */
  audio_attr_t vtsm_audio_attr;
  audio_attr_t zero_2[7];  
  uint8_t  zero_3[16];
  uint8_t  zero_4;
  uint8_t  nr_of_vtsm_subp_streams; /* should be 0 or 1 */
  subp_attr_t vtsm_subp_attr;
  subp_attr_t zero_5[27];
  
  uint8_t  zero_6[2];
  
  video_attr_t vtstt_vobs_video_attr;
  uint8_t  zero_7;
  uint8_t  nr_of_vtstt_audio_streams;
  audio_attr_t vtstt_audio_attr[8];
  uint8_t  zero_8[16];
  uint8_t  zero_9;
  uint8_t  nr_of_vtstt_subp_streams;
  subp_attr_t vtstt_subp_attr[32];
} ATTRIBUTE_PACKED vts_attributes_t;
%extend vts_attributes_t {
    void set_zero_2(int index, audio_attr_t obj)
    {
        $self->zero_2[index] = obj;
    }
    void set_zero_3(int index, uint8_t obj)
    {
        $self->zero_3[index] = obj;
    }
    void set_zero_5(int index, subp_attr_t obj)
    {
        $self->zero_5[index] = obj;
    }
    void set_zero_6(int index, uint8_t obj)
    {
        $self->zero_6[index] = obj;
    }
    void set_vtstt_audio_attr(int index, audio_attr_t obj)
    {
        $self->vtstt_audio_attr[index] = obj;
    }
    void set_vtstt_subp_attr(int index, subp_attr_t obj)
    {
        $self->vtstt_subp_attr[index] = obj;
    }
    void set_zero_8(int index, uint8_t obj)
    {
        $self->zero_8[index] = obj;
    }
};


/**
 * vts_atrt_t
 */
%feature("autodoc", "Video Title Set Attribute Table.") vts_atrt_t;
typedef struct {
  uint16_t nr_of_vtss;
  uint16_t zero_1;
  uint32_t last_byte;
  vts_attributes_t *vts;
  uint32_t *vts_atrt_offsets; /* offsets table for each vts_attributes */
} ATTRIBUTE_PACKED vts_atrt_t;
%extend vts_atrt_t {
    vts_attributes_t vts_get(int index)
    {
        return $self->vts[index];
    }
    void set_vts(PyObject *obj)
    {
        int i, dim=0;
        int ret=0;
        void *ptr=0;
        vts_attributes_t *temp=0;
        if (!PyList_Check(obj)) {
            return;
        }
        dim = PyList_Size(obj);
        if (dim <= 0) return;
        temp = (vts_attributes_t *) malloc(dim * sizeof(vts_attributes_t));
        for (i = 0; i < dim; i++) {
            PyObject *o = PyList_GetItem(obj, i);
            ret = SWIG_ConvertPtr(o, &ptr, SWIGTYPE_p_vts_attributes_t, 0 |  0 );
            if (!SWIG_IsOK(ret)) {
                free(temp);
                return;
            } else {
                temp[i] = *((vts_attributes_t *) ptr);
            }
        }
        if ($self->vts)
            free($self->vts);
        $self->vts = temp;
    }
    uint32_t vts_atrt_offsets_get(int index)
    {
        return $self->vts_atrt_offsets[index];
    }
    void set_vts_atrt_offsets(PyObject *obj)
    {
        int i, dim=0;
        int ret=0;
        uint32_t *temp=0;
        if (!PyList_Check(obj)) {
            return;
        }
        dim = PyList_Size(obj);
        if (dim <= 0) return;
        temp = (uint32_t *) malloc(dim * sizeof(uint32_t));
        for (i = 0; i < dim; i++) {
            PyObject *o = PyList_GetItem(obj, i);
            if (PyInt_Check(o)) {
                temp[i] = (uint32_t) PyInt_AsLong(o);
            } else {
                free(temp);
                return;
            }
        }
        if ($self->vts_atrt_offsets)
            free($self->vts_atrt_offsets);
        $self->vts_atrt_offsets = temp;
    }
};


/**
 * txtdt_t
 */
%feature("autodoc", "Text Data.") txtdt_t;
typedef struct {
  uint32_t last_byte;    /* offsets are relative here */
  uint16_t offsets[100]; /* == nr_of_srpts + 1 (first is disc title) */
#if 0  
  uint16_t unknown; /* 0x48 ?? 0x48 words (16bit) info following */
  uint16_t zero_1;
  
  uint8_t type_of_info; /* ?? 01 == disc, 02 == Title, 04 == Title part */
  uint8_t unknown1;
  uint8_t unknown2;
  uint8_t unknown3;
  uint8_t unknown4; /* ?? allways 0x30 language?, text format? */
  uint8_t unknown5;
  uint16_t offset; /* from first */
  
  char text[12]; /* ended by 0x09 */
#endif
} ATTRIBUTE_PACKED txtdt_t;
%extend txtdt_t {
    void set_offsets(int index, uint16_t obj)
    {
        $self->offsets[index] = obj;
    }
};


/**
 * txtdt_lu_t
 */
%feature("autodoc", "Text Data Language Unit.") txtdt_lu_t;
typedef struct {
  uint16_t lang_code;
  uint16_t unknown;      /* 0x0001, title 1? disc 1? side 1? */
  uint32_t txtdt_start_byte;  /* prt, rel start of vmg_txtdt_mgi  */
  txtdt_t  *txtdt;
} ATTRIBUTE_PACKED txtdt_lu_t;


/**
 * txtdt_mgi_t
 */
%feature("autodoc", "Text Data Manager Information.") txtdt_mgi_t;
typedef struct {
  char disc_name[14];            /* how many bytes?? */
  uint16_t nr_of_language_units; /* 32bit??          */
  uint32_t last_byte;
  txtdt_lu_t *lu;
} ATTRIBUTE_PACKED txtdt_mgi_t;
%extend txtdt_mgi_t {
    txtdt_lu_t lu_get(int index)
    {
        return $self->lu[index];
    }
    void set_lu(PyObject *obj)
    {
        int i, dim=0;
        int ret=0;
        void *ptr=0;
        txtdt_lu_t *temp=0;
        if (!PyList_Check(obj)) {
            return;
        }
        dim = PyList_Size(obj);
        if (dim <= 0) return;
        temp = (txtdt_lu_t *) malloc(dim * sizeof(txtdt_lu_t));
        for (i = 0; i < dim; i++) {
            PyObject *o = PyList_GetItem(obj, i);
            ret = SWIG_ConvertPtr(o, &ptr, SWIGTYPE_p_txtdt_lu_t, 0 |  0 );
            if (!SWIG_IsOK(ret)) {
                free(temp);
                return;
            } else {
                temp[i] = *((txtdt_lu_t *) ptr);
            }
        }
        if ($self->lu)
            free($self->lu);
        $self->lu = temp;
    }
};


/**
 * VTS
 *
 * Structures relating to the Video Title Set (VTS).
 */


/**
 * vtsi_mat_t
 */
%feature("autodoc", "Video Title Set Information Management Table.") vtsi_mat_t;
typedef struct {
  char vts_identifier[12];
  uint32_t vts_last_sector;
  uint8_t  zero_1[12];
  uint32_t vtsi_last_sector;
  uint8_t  zero_2;
  uint8_t  specification_version;
  uint32_t vts_category;
  uint16_t zero_3;
  uint16_t zero_4;
  uint8_t  zero_5;
  uint8_t  zero_6[19];
  uint16_t zero_7;
  uint8_t  zero_8[32];
  uint64_t zero_9;
  uint8_t  zero_10[24];
  uint32_t vtsi_last_byte;
  uint32_t zero_11;
  uint8_t  zero_12[56];
  uint32_t vtsm_vobs;       /* sector */
  uint32_t vtstt_vobs;      /* sector */
  uint32_t vts_ptt_srpt;    /* sector */
  uint32_t vts_pgcit;       /* sector */
  uint32_t vtsm_pgci_ut;    /* sector */
  uint32_t vts_tmapt;       /* sector */
  uint32_t vtsm_c_adt;      /* sector */
  uint32_t vtsm_vobu_admap; /* sector */
  uint32_t vts_c_adt;       /* sector */
  uint32_t vts_vobu_admap;  /* sector */
  uint8_t  zero_13[24];
  
  video_attr_t vtsm_video_attr;
  uint8_t  zero_14;
  uint8_t  nr_of_vtsm_audio_streams; /* should be 0 or 1 */
  audio_attr_t vtsm_audio_attr;
  audio_attr_t zero_15[7];
  uint8_t  zero_16[17];
  uint8_t  nr_of_vtsm_subp_streams; /* should be 0 or 1 */
  subp_attr_t vtsm_subp_attr;
  subp_attr_t zero_17[27];
  uint8_t  zero_18[2];
  
  video_attr_t vts_video_attr;
  uint8_t  zero_19;
  uint8_t  nr_of_vts_audio_streams;
  audio_attr_t vts_audio_attr[8];
  uint8_t  zero_20[17];
  uint8_t  nr_of_vts_subp_streams;
  subp_attr_t vts_subp_attr[32];
  uint16_t zero_21;
  multichannel_ext_t vts_mu_audio_attr[8];
  /* XXX: how much 'padding' here, if any? */
} ATTRIBUTE_PACKED vtsi_mat_t;
%extend vtsi_mat_t {
    void set_zero_1(int index, uint8_t obj)
    {
        $self->zero_1[index] = obj;
    }
    void set_zero_6(int index, uint8_t obj)
    {
        $self->zero_6[index] = obj;
    }
    void set_zero_8(int index, uint8_t obj)
    {
        $self->zero_8[index] = obj;
    }
    void set_zero_10(int index, uint8_t obj)
    {
        $self->zero_10[index] = obj;
    }
    void set_zero_12(int index, uint8_t obj)
    {
        $self->zero_12[index] = obj;
    }
    void set_zero_13(int index, uint8_t obj)
    {
        $self->zero_13[index] = obj;
    }
    void set_zero_16(int index, uint8_t obj)
    {
        $self->zero_16[index] = obj;
    }
    void set_zero_18(int index, uint8_t obj)
    {
        $self->zero_18[index] = obj;
    }
    void set_zero_20(int index, uint8_t obj)
    {
        $self->zero_20[index] = obj;
    }
    void set_zero_17(int index, subp_attr_t obj)
    {
        $self->zero_17[index] = obj;
    }
    void set_vts_audio_attr(int index, audio_attr_t obj)
    {
        $self->vts_audio_attr[index] = obj;
    }
    void set_vts_subp_attr(int index, subp_attr_t obj)
    {
        $self->vts_subp_attr[index] = obj;
    }
    void set_vts_mu_audio_attr(int index, multichannel_ext_t obj)
    {
        $self->vts_mu_audio_attr[index] = obj;
    }
};


/**
 * ptt_info_t
 */
%feature("autodoc", "PartOfTitle Unit Information.") ptt_info_t;
typedef struct {
  uint16_t pgcn;
  uint16_t pgn;
} ATTRIBUTE_PACKED ptt_info_t;


/**
 * ttu_t
 */
%feature("autodoc", "PartOfTitle Information.") ttu_t;
typedef struct {
  uint16_t nr_of_ptts;
  ptt_info_t *ptt;
} ATTRIBUTE_PACKED ttu_t;
%extend ttu_t {
    ptt_info_t ptt_get(int index)
    {
        return $self->ptt[index];
    }
    void set_ptt(PyObject *obj)
    {
        int i, dim=0;
        int ret=0;
        void *ptr=0;
        ptt_info_t *temp=0;
        if (!PyList_Check(obj)) {
            return;
        }
        dim = PyList_Size(obj);
        if (dim <= 0) return;
        temp = (ptt_info_t *) malloc(dim * sizeof(ptt_info_t));
        for (i = 0; i < dim; i++) {
            PyObject *o = PyList_GetItem(obj, i);
            ret = SWIG_ConvertPtr(o, &ptr, SWIGTYPE_p_ptt_info_t, 0 |  0 );
            if (!SWIG_IsOK(ret)) {
                free(temp);
                return;
            } else {
                temp[i] = *((ptt_info_t *) ptr);
            }
        }
        if ($self->ptt)
            free($self->ptt);
        $self->ptt = temp;
    }
};


/**
 * vts_ptt_srpt_t
 */
%feature("autodoc", "PartOfTitle Search Pointer Table.") vts_ptt_srpt_t;
typedef struct {
  uint16_t nr_of_srpts;
  uint16_t zero_1;
  uint32_t last_byte;
  ttu_t  *title;
  uint32_t *ttu_offset; /* offset table for each ttu */
} ATTRIBUTE_PACKED vts_ptt_srpt_t;
%extend vts_ptt_srpt_t {
    ttu_t title_get(int index)
    {
        return $self->title[index];
    }
    void set_title(PyObject *obj)
    {
        int i, dim=0;
        int ret=0;
        void *ptr=0;
        ttu_t *temp=0;
        if (!PyList_Check(obj)) {
            return;
        }
        dim = PyList_Size(obj);
        if (dim <= 0) return;
        temp = (ttu_t *) malloc(dim * sizeof(ttu_t));
        for (i = 0; i < dim; i++) {
            PyObject *o = PyList_GetItem(obj, i);
            ret = SWIG_ConvertPtr(o, &ptr, SWIGTYPE_p_ttu_t, 0 |  0 );
            if (!SWIG_IsOK(ret)) {
                free(temp);
                return;
            } else {
                temp[i] = *((ttu_t *) ptr);
            }
        }
        if ($self->title)
            free($self->title);
        $self->title = temp;
    }
    uint32_t ttu_offset_get(int index)
    {
        return $self->ttu_offset[index];
    }
    void set_ttu_offset(PyObject *obj)
    {
        int i, dim=0;
        int ret=0;
        uint32_t *temp=0;
        if (!PyList_Check(obj)) {
            return;
        }
        dim = PyList_Size(obj);
        if (dim <= 0) return;
        temp = (uint32_t *) malloc(dim * sizeof(uint32_t));
        for (i = 0; i < dim; i++) {
            PyObject *o = PyList_GetItem(obj, i);
            if (PyInt_Check(o)) {
                temp[i] = (uint32_t) PyInt_AsLong(o);
            } else {
                free(temp);
                return;
            }
        }
        if ($self->ttu_offset)
            free($self->ttu_offset);
        $self->ttu_offset = temp;
    }
};


/**
 * Time Map Entry.
 */
/* Should this be bit field at all or just the uint32_t? */
typedef uint32_t map_ent_t;


/**
 * vts_tmap_t
 */
%feature("autodoc", "Time Map.") vts_tmap_t;
typedef struct {
  uint8_t  tmu;   /* Time unit, in seconds */
  uint8_t  zero_1;
  uint16_t nr_of_entries;
  map_ent_t *map_ent;
} ATTRIBUTE_PACKED vts_tmap_t;
%extend vts_tmap_t {
    map_ent_t map_ent_get(int index)
    {
        return $self->map_ent[index];
    }
    void set_map_ent(PyObject *obj)
    {
        int i, dim=0;
        int ret=0;
        uint32_t *temp=0;
        if (!PyList_Check(obj)) {
            return;
        }
        dim = PyList_Size(obj);
        if (dim <= 0) return;
        temp = (uint32_t *) malloc(dim * sizeof(uint32_t));
        for (i = 0; i < dim; i++) {
            PyObject *o = PyList_GetItem(obj, i);
            if (PyInt_Check(o)) {
                temp[i] = (uint32_t) PyInt_AsLong(o);
            } else {
                free(temp);
                return;
            }
        }
        if ($self->map_ent)
            free($self->map_ent);
        $self->map_ent = temp;
    }
};


/**
 * vts_tmapt_t
 */
%feature("autodoc", "Time Map Table.") vts_tmapt_t;
typedef struct {
  uint16_t nr_of_tmaps;
  uint16_t zero_1;
  uint32_t last_byte;
  vts_tmap_t *tmap;
  uint32_t *tmap_offset; /* offset table for each tmap */
} ATTRIBUTE_PACKED vts_tmapt_t;
%extend vts_tmapt_t {
    vts_tmap_t tmap_get(int index)
    {
        return $self->tmap[index];
    }
    void set_tmap(PyObject *obj)
    {
        int i, dim=0;
        int ret=0;
        void *ptr=0;
        vts_tmap_t *temp=0;
        if (!PyList_Check(obj)) {
            return;
        }
        dim = PyList_Size(obj);
        if (dim <= 0) return;
        temp = (vts_tmap_t *) malloc(dim * sizeof(vts_tmap_t));
        for (i = 0; i < dim; i++) {
            PyObject *o = PyList_GetItem(obj, i);
            ret = SWIG_ConvertPtr(o, &ptr, SWIGTYPE_p_vts_tmap_t, 0 |  0 );
            if (!SWIG_IsOK(ret)) {
                free(temp);
                return;
            } else {
                temp[i] = *((vts_tmap_t *) ptr);
            }
        }
        if ($self->tmap)
            free($self->tmap);
        $self->tmap = temp;
    }
    uint32_t tmap_offset_get(int index)
    {
        return $self->tmap_offset[index];
    }
    void set_tmap_offset(PyObject *obj)
    {
        int i, dim=0;
        int ret=0;
        uint32_t *temp=0;
        if (!PyList_Check(obj)) {
            return;
        }
        dim = PyList_Size(obj);
        if (dim <= 0) return;
        temp = (uint32_t *) malloc(dim * sizeof(uint32_t));
        for (i = 0; i < dim; i++) {
            PyObject *o = PyList_GetItem(obj, i);
            if (PyInt_Check(o)) {
                temp[i] = (uint32_t) PyInt_AsLong(o);
            } else {
                free(temp);
                return;
            }
        }
        if ($self->tmap_offset)
            free($self->tmap_offset);
        $self->tmap_offset = temp;
    }
};


#if PRAGMA_PACK
#pragma pack()
#endif




/**
 * ifo_handle_t
 */
%feature("autodoc", "The following structure defines an IFO file.\n\n\
The structure is divided into two parts, the VMGI, or Video Manager Information,\n\
which is read from the VIDEO_TS.[IFO,BUP] file, and the VTSI,\n\
or Video Title Set Information, which is read in from the VTS_XX_0.[IFO,BUP] files.") ifo_handle_t;
typedef struct {
  dvd_file_t *file;
  
  /* VMGI */
  vmgi_mat_t     *vmgi_mat;
  tt_srpt_t      *tt_srpt;
  pgc_t          *first_play_pgc;    
  ptl_mait_t     *ptl_mait;
  vts_atrt_t     *vts_atrt;
  txtdt_mgi_t    *txtdt_mgi;
  
  /* Common */
  pgci_ut_t      *pgci_ut;
  c_adt_t        *menu_c_adt;
  vobu_admap_t   *menu_vobu_admap;
  
  /* VTSI */
  vtsi_mat_t     *vtsi_mat;
  vts_ptt_srpt_t *vts_ptt_srpt;
  pgcit_t        *vts_pgcit;
  vts_tmapt_t    *vts_tmapt;
  c_adt_t        *vts_c_adt;
  vobu_admap_t   *vts_vobu_admap;
} ifo_handle_t;

